@@ -155,8 +155,15 @@ GST_DEBUG_CATEGORY_STATIC (flacenc_debug);
       NULL,                                                                     \
       NULL                                                                      \
     };                                                                          \
+    static const GInterfaceInfo preset_info = {                                 \
+      NULL,                                                                     \
+      NULL,                                                                     \
+      NULL                                                                      \
+    };                                                                          \
     g_type_add_interface_static (type, GST_TYPE_TAG_SETTER,                     \
                                  &tag_setter_info);                             \
+    g_type_add_interface_static (type, GST_TYPE_PRESET,                         \
+                                 &preset_info);                                 \
   }G_STMT_END
 
 GST_BOILERPLATE_FULL (GstFlacEnc, gst_flac_enc, GstElement, GST_TYPE_ELEMENT,
@@ -178,18 +185,6 @@ static void gst_flac_enc_get_property (GObject * object, guint prop_id,
 static GstStateChangeReturn gst_flac_enc_change_state (GstElement * element,
     GstStateChange transition);
 
-#ifdef LEGACY_FLAC
-static FLAC__StreamEncoderWriteStatus
-gst_flac_enc_write_callback (const FLAC__SeekableStreamEncoder * encoder,
-    const FLAC__byte buffer[], unsigned bytes,
-    unsigned samples, unsigned current_frame, void *client_data);
-static FLAC__SeekableStreamEncoderSeekStatus
-gst_flac_enc_seek_callback (const FLAC__SeekableStreamEncoder * encoder,
-    FLAC__uint64 absolute_byte_offset, void *client_data);
-static FLAC__SeekableStreamEncoderTellStatus
-gst_flac_enc_tell_callback (const FLAC__SeekableStreamEncoder * encoder,
-    FLAC__uint64 * absolute_byte_offset, void *client_data);
-#else
 static FLAC__StreamEncoderWriteStatus
 gst_flac_enc_write_callback (const FLAC__StreamEncoder * encoder,
     const FLAC__byte buffer[], size_t bytes,
@@ -200,7 +195,6 @@ gst_flac_enc_seek_callback (const FLAC__StreamEncoder * encoder,
 static FLAC__StreamEncoderTellStatus
 gst_flac_enc_tell_callback (const FLAC__StreamEncoder * encoder,
     FLAC__uint64 * absolute_byte_offset, void *client_data);
-#endif
 
 typedef struct
 {
@@ -392,11 +386,7 @@ gst_flac_enc_init (GstFlacEnc * flacenc, GstFlacEncClass * klass)
   gst_pad_use_fixed_caps (flacenc->srcpad);
   gst_element_add_pad (GST_ELEMENT (flacenc), flacenc->srcpad);
 
-#ifdef LEGACY_FLAC
-  flacenc->encoder = FLAC__seekable_stream_encoder_new ();
-#else
   flacenc->encoder = FLAC__stream_encoder_new ();
-#endif
 
   flacenc->offset = 0;
   flacenc->samples_written = 0;
@@ -414,11 +404,7 @@ gst_flac_enc_finalize (GObject * object)
   GstFlacEnc *flacenc = GST_FLAC_ENC (object);
 
   gst_tag_list_free (flacenc->tags);
-#ifdef LEGACY_FLAC
-  FLAC__seekable_stream_encoder_delete (flacenc->encoder);
-#else
   FLAC__stream_encoder_delete (flacenc->encoder);
-#endif
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -463,13 +449,9 @@ gst_flac_enc_set_metadata (GstFlacEnc * flacenc)
       FLAC__metadata_object_new (FLAC__METADATA_TYPE_VORBIS_COMMENT);
   gst_tag_list_foreach (copy, add_one_tag, flacenc);
 
-#ifdef LEGACY_FLAC
-  if (FLAC__seekable_stream_encoder_set_metadata (flacenc->encoder,
-          flacenc->meta, 1) != true)
-#else
   if (FLAC__stream_encoder_set_metadata (flacenc->encoder,
           flacenc->meta, 1) != true)
-#endif
+
     g_warning ("Dude, i'm already initialized!");
   gst_tag_list_free (copy);
 }
@@ -606,24 +588,13 @@ gst_flac_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   GstFlacEnc *flacenc;
   GstStructure *structure;
   guint64 total_samples = GST_CLOCK_TIME_NONE;
-
-#ifdef LEGACY_FLAC
-  FLAC__SeekableStreamEncoderState state;
-#else
   FLAC__StreamEncoderInitStatus init_status;
-#endif
   gint depth, chans, rate, width;
 
   flacenc = GST_FLAC_ENC (gst_pad_get_parent (pad));
 
-#ifdef LEGACY_FLAC
-  if (FLAC__seekable_stream_encoder_get_state (flacenc->encoder) !=
-      FLAC__SEEKABLE_STREAM_ENCODER_UNINITIALIZED)
-#else
   if (FLAC__stream_encoder_get_state (flacenc->encoder) !=
       FLAC__STREAM_ENCODER_UNINITIALIZED)
-#endif
-
     goto encoder_already_initialized;
 
   structure = gst_caps_get_structure (caps, 0);
@@ -652,27 +623,6 @@ gst_flac_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
 
   total_samples = gst_flac_enc_query_peer_total_samples (flacenc, pad);
 
-#ifdef LEGACY_FLAC
-  FLAC__seekable_stream_encoder_set_bits_per_sample (flacenc->encoder,
-      flacenc->depth);
-  FLAC__seekable_stream_encoder_set_sample_rate (flacenc->encoder,
-      flacenc->sample_rate);
-  FLAC__seekable_stream_encoder_set_channels (flacenc->encoder,
-      flacenc->channels);
-
-  if (total_samples != GST_CLOCK_TIME_NONE)
-    FLAC__seekable_stream_encoder_set_total_samples_estimate (flacenc->encoder,
-        total_samples);
-
-  FLAC__seekable_stream_encoder_set_write_callback (flacenc->encoder,
-      gst_flac_enc_write_callback);
-  FLAC__seekable_stream_encoder_set_seek_callback (flacenc->encoder,
-      gst_flac_enc_seek_callback);
-  FLAC__seekable_stream_encoder_set_tell_callback (flacenc->encoder,
-      gst_flac_enc_tell_callback);
-
-  FLAC__seekable_stream_encoder_set_client_data (flacenc->encoder, flacenc);
-#else
   FLAC__stream_encoder_set_bits_per_sample (flacenc->encoder, flacenc->depth);
   FLAC__stream_encoder_set_sample_rate (flacenc->encoder, flacenc->sample_rate);
   FLAC__stream_encoder_set_channels (flacenc->encoder, flacenc->channels);
@@ -680,21 +630,14 @@ gst_flac_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   if (total_samples != GST_CLOCK_TIME_NONE)
     FLAC__stream_encoder_set_total_samples_estimate (flacenc->encoder,
         total_samples);
-#endif
 
   gst_flac_enc_set_metadata (flacenc);
 
-#ifdef LEGACY_FLAC
-  state = FLAC__seekable_stream_encoder_init (flacenc->encoder);
-  if (state != FLAC__STREAM_ENCODER_OK)
-    goto failed_to_initialize;
-#else
   init_status = FLAC__stream_encoder_init_stream (flacenc->encoder,
       gst_flac_enc_write_callback, gst_flac_enc_seek_callback,
       gst_flac_enc_tell_callback, NULL, flacenc);
   if (init_status != FLAC__STREAM_ENCODER_INIT_STATUS_OK)
     goto failed_to_initialize;
-#endif
 
   gst_object_unref (flacenc);
 
@@ -728,18 +671,6 @@ gst_flac_enc_update_quality (GstFlacEnc * flacenc, gint quality)
 {
   flacenc->quality = quality;
 
-#ifdef LEGACY_FLAC
-#define DO_UPDATE(name, val, str)                                               \
-  G_STMT_START {                                                                \
-    if (FLAC__seekable_stream_encoder_get_##name (flacenc->encoder) !=          \
-        flacenc_params[quality].val) {                                          \
-      FLAC__seekable_stream_encoder_set_##name (flacenc->encoder,               \
-          flacenc_params[quality].val);                                         \
-      g_object_notify (G_OBJECT (flacenc), str);                                \
-    }                                                                           \
-  } G_STMT_END
-
-#else
 #define DO_UPDATE(name, val, str)                                               \
   G_STMT_START {                                                                \
     if (FLAC__stream_encoder_get_##name (flacenc->encoder) !=                   \
@@ -750,8 +681,6 @@ gst_flac_enc_update_quality (GstFlacEnc * flacenc, gint quality)
     }                                                                           \
   } G_STMT_END
 
-#endif
-
   g_object_freeze_notify (G_OBJECT (flacenc));
 
   if (flacenc->channels == 2 || flacenc->channels == 0) {
@@ -781,15 +710,9 @@ gst_flac_enc_update_quality (GstFlacEnc * flacenc, gint quality)
   return TRUE;
 }
 
-#ifdef LEGACY_FLAC
-static FLAC__SeekableStreamEncoderSeekStatus
-gst_flac_enc_seek_callback (const FLAC__SeekableStreamEncoder * encoder,
-    FLAC__uint64 absolute_byte_offset, void *client_data)
-#else
 static FLAC__StreamEncoderSeekStatus
 gst_flac_enc_seek_callback (const FLAC__StreamEncoder * encoder,
     FLAC__uint64 absolute_byte_offset, void *client_data)
-#endif
 {
   GstFlacEnc *flacenc;
   GstEvent *event;
@@ -798,11 +721,8 @@ gst_flac_enc_seek_callback (const FLAC__StreamEncoder * encoder,
   flacenc = GST_FLAC_ENC (client_data);
 
   if (flacenc->stopped)
-#ifdef LEGACY_FLAC
-    return FLAC__SEEKABLE_STREAM_ENCODER_SEEK_STATUS_OK;
-#else
     return FLAC__STREAM_ENCODER_SEEK_STATUS_OK;
-#endif
+
   event = gst_event_new_new_segment (TRUE, 1.0, GST_FORMAT_BYTES,
       absolute_byte_offset, GST_BUFFER_OFFSET_NONE, 0);
 
@@ -817,12 +737,7 @@ gst_flac_enc_seek_callback (const FLAC__StreamEncoder * encoder,
     } else {
       GST_DEBUG ("Seek to %" G_GUINT64_FORMAT " %s", absolute_byte_offset,
           "failed");
-#ifdef LEGACY_FLAC
-      return FLAC__SEEKABLE_STREAM_ENCODER_SEEK_STATUS_ERROR;
-#else
       return FLAC__STREAM_ENCODER_SEEK_STATUS_UNSUPPORTED;
-#endif
-
     }
   } else {
     GST_DEBUG ("Seek to %" G_GUINT64_FORMAT " failed (no peer pad)",
@@ -830,11 +745,7 @@ gst_flac_enc_seek_callback (const FLAC__StreamEncoder * encoder,
   }
 
   flacenc->offset = absolute_byte_offset;
-#ifdef LEGACY_FLAC
-  return FLAC__SEEKABLE_STREAM_ENCODER_SEEK_STATUS_OK;
-#else
   return FLAC__STREAM_ENCODER_SEEK_STATUS_OK;
-#endif
 }
 
 static void
@@ -959,17 +870,10 @@ push_headers:
   gst_caps_unref (caps);
 }
 
-#ifdef LEGACY_FLAC
-static FLAC__StreamEncoderWriteStatus
-gst_flac_enc_write_callback (const FLAC__SeekableStreamEncoder * encoder,
-    const FLAC__byte buffer[], unsigned bytes,
-    unsigned samples, unsigned current_frame, void *client_data)
-#else
 static FLAC__StreamEncoderWriteStatus
 gst_flac_enc_write_callback (const FLAC__StreamEncoder * encoder,
     const FLAC__byte buffer[], size_t bytes,
     unsigned samples, unsigned current_frame, void *client_data)
-#endif
 {
   GstFlowReturn ret = GST_FLOW_OK;
   GstFlacEnc *flacenc;
@@ -1048,25 +952,15 @@ out:
   return FLAC__STREAM_ENCODER_WRITE_STATUS_OK;
 }
 
-#ifdef LEGACY_FLAC
-static FLAC__SeekableStreamEncoderTellStatus
-gst_flac_enc_tell_callback (const FLAC__SeekableStreamEncoder * encoder,
-    FLAC__uint64 * absolute_byte_offset, void *client_data)
-#else
 static FLAC__StreamEncoderTellStatus
 gst_flac_enc_tell_callback (const FLAC__StreamEncoder * encoder,
     FLAC__uint64 * absolute_byte_offset, void *client_data)
-#endif
 {
   GstFlacEnc *flacenc = GST_FLAC_ENC (client_data);
 
   *absolute_byte_offset = flacenc->offset;
 
-#ifdef LEGACY_FLAC
-  return FLAC__SEEKABLE_STREAM_ENCODER_TELL_STATUS_OK;
-#else
   return FLAC__STREAM_ENCODER_TELL_STATUS_OK;
-#endif
 }
 
 static gboolean
@@ -1110,17 +1004,14 @@ gst_flac_enc_sink_event (GstPad * pad, GstEvent * event)
       break;
     }
     case GST_EVENT_EOS:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_finish (flacenc->encoder);
-#else
       FLAC__stream_encoder_finish (flacenc->encoder);
-#endif
       ret = gst_pad_event_default (pad, event);
       break;
     case GST_EVENT_TAG:
       if (flacenc->tags) {
         gst_event_parse_tag (event, &taglist);
-        gst_tag_list_insert (flacenc->tags, taglist, GST_TAG_MERGE_REPLACE);
+        gst_tag_list_insert (flacenc->tags, taglist,
+            gst_tag_setter_get_tag_merge_mode (GST_TAG_SETTER (flacenc)));
       } else {
         g_assert_not_reached ();
       }
@@ -1230,13 +1121,8 @@ gst_flac_enc_chain (GstPad * pad, GstBuffer * buffer)
 
   gst_buffer_unref (buffer);
 
-#ifdef LEGACY_FLAC
-  res = FLAC__seekable_stream_encoder_process_interleaved (flacenc->encoder,
-      (const FLAC__int32 *) data, samples / flacenc->channels);
-#else
   res = FLAC__stream_encoder_process_interleaved (flacenc->encoder,
       (const FLAC__int32 *) data, samples / flacenc->channels);
-#endif
 
   g_free (data);
 
@@ -1263,112 +1149,52 @@ gst_flac_enc_set_property (GObject * object, guint prop_id,
       gst_flac_enc_update_quality (this, g_value_get_enum (value));
       break;
     case PROP_STREAMABLE_SUBSET:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_set_streamable_subset (this->encoder,
-          g_value_get_boolean (value));
-#else
       FLAC__stream_encoder_set_streamable_subset (this->encoder,
           g_value_get_boolean (value));
-#endif
       break;
     case PROP_MID_SIDE_STEREO:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_set_do_mid_side_stereo (this->encoder,
-          g_value_get_boolean (value));
-#else
       FLAC__stream_encoder_set_do_mid_side_stereo (this->encoder,
           g_value_get_boolean (value));
-#endif
       break;
     case PROP_LOOSE_MID_SIDE_STEREO:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_set_loose_mid_side_stereo (this->encoder,
-          g_value_get_boolean (value));
-#else
       FLAC__stream_encoder_set_loose_mid_side_stereo (this->encoder,
           g_value_get_boolean (value));
-#endif
       break;
     case PROP_BLOCKSIZE:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_set_blocksize (this->encoder,
-          g_value_get_uint (value));
-#else
       FLAC__stream_encoder_set_blocksize (this->encoder,
           g_value_get_uint (value));
-#endif
       break;
     case PROP_MAX_LPC_ORDER:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_set_max_lpc_order (this->encoder,
-          g_value_get_uint (value));
-#else
       FLAC__stream_encoder_set_max_lpc_order (this->encoder,
           g_value_get_uint (value));
-#endif
       break;
     case PROP_QLP_COEFF_PRECISION:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_set_qlp_coeff_precision (this->encoder,
-          g_value_get_uint (value));
-#else
       FLAC__stream_encoder_set_qlp_coeff_precision (this->encoder,
           g_value_get_uint (value));
-#endif
       break;
     case PROP_QLP_COEFF_PREC_SEARCH:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_set_do_qlp_coeff_prec_search (this->encoder,
-          g_value_get_boolean (value));
-#else
       FLAC__stream_encoder_set_do_qlp_coeff_prec_search (this->encoder,
           g_value_get_boolean (value));
-#endif
       break;
     case PROP_ESCAPE_CODING:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_set_do_escape_coding (this->encoder,
-          g_value_get_boolean (value));
-#else
       FLAC__stream_encoder_set_do_escape_coding (this->encoder,
           g_value_get_boolean (value));
-#endif
       break;
     case PROP_EXHAUSTIVE_MODEL_SEARCH:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_set_do_exhaustive_model_search
-          (this->encoder, g_value_get_boolean (value));
-#else
       FLAC__stream_encoder_set_do_exhaustive_model_search (this->encoder,
           g_value_get_boolean (value));
-#endif
       break;
     case PROP_MIN_RESIDUAL_PARTITION_ORDER:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_set_min_residual_partition_order
-          (this->encoder, g_value_get_uint (value));
-#else
       FLAC__stream_encoder_set_min_residual_partition_order (this->encoder,
           g_value_get_uint (value));
-#endif
       break;
     case PROP_MAX_RESIDUAL_PARTITION_ORDER:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_set_max_residual_partition_order
-          (this->encoder, g_value_get_uint (value));
-#else
       FLAC__stream_encoder_set_max_residual_partition_order (this->encoder,
           g_value_get_uint (value));
-#endif
       break;
     case PROP_RICE_PARAMETER_SEARCH_DIST:
-#ifdef LEGACY_FLAC
-      FLAC__seekable_stream_encoder_set_rice_parameter_search_dist
-          (this->encoder, g_value_get_uint (value));
-#else
       FLAC__stream_encoder_set_rice_parameter_search_dist (this->encoder,
           g_value_get_uint (value));
-#endif
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1391,121 +1217,54 @@ gst_flac_enc_get_property (GObject * object, guint prop_id,
       g_value_set_enum (value, this->quality);
       break;
     case PROP_STREAMABLE_SUBSET:
-#ifdef LEGACY_FLAC
-      g_value_set_boolean (value,
-          FLAC__seekable_stream_encoder_get_streamable_subset (this->encoder));
-#else
       g_value_set_boolean (value,
           FLAC__stream_encoder_get_streamable_subset (this->encoder));
-#endif
       break;
     case PROP_MID_SIDE_STEREO:
-#ifdef LEGACY_FLAC
-      g_value_set_boolean (value,
-          FLAC__seekable_stream_encoder_get_do_mid_side_stereo (this->encoder));
-#else
       g_value_set_boolean (value,
           FLAC__stream_encoder_get_do_mid_side_stereo (this->encoder));
-#endif
       break;
     case PROP_LOOSE_MID_SIDE_STEREO:
-#ifdef LEGACY_FLAC
-      g_value_set_boolean (value,
-          FLAC__seekable_stream_encoder_get_loose_mid_side_stereo
-          (this->encoder));
-#else
       g_value_set_boolean (value,
           FLAC__stream_encoder_get_loose_mid_side_stereo (this->encoder));
-#endif
       break;
     case PROP_BLOCKSIZE:
-#ifdef LEGACY_FLAC
-      g_value_set_uint (value,
-          FLAC__seekable_stream_encoder_get_blocksize (this->encoder));
-#else
       g_value_set_uint (value,
           FLAC__stream_encoder_get_blocksize (this->encoder));
-#endif
       break;
     case PROP_MAX_LPC_ORDER:
-#ifdef LEGACY_FLAC
-      g_value_set_uint (value,
-          FLAC__seekable_stream_encoder_get_max_lpc_order (this->encoder));
-#else
       g_value_set_uint (value,
           FLAC__stream_encoder_get_max_lpc_order (this->encoder));
-#endif
       break;
     case PROP_QLP_COEFF_PRECISION:
-#ifdef LEGACY_FLAC
-      g_value_set_uint (value,
-          FLAC__seekable_stream_encoder_get_qlp_coeff_precision
-          (this->encoder));
-#else
       g_value_set_uint (value,
           FLAC__stream_encoder_get_qlp_coeff_precision (this->encoder));
-#endif
       break;
     case PROP_QLP_COEFF_PREC_SEARCH:
-#ifdef LEGACY_FLAC
-      g_value_set_boolean (value,
-          FLAC__seekable_stream_encoder_get_do_qlp_coeff_prec_search
-          (this->encoder));
-#else
       g_value_set_boolean (value,
           FLAC__stream_encoder_get_do_qlp_coeff_prec_search (this->encoder));
-#endif
       break;
     case PROP_ESCAPE_CODING:
-#ifdef LEGACY_FLAC
-      g_value_set_boolean (value,
-          FLAC__seekable_stream_encoder_get_do_escape_coding (this->encoder));
-#else
       g_value_set_boolean (value,
           FLAC__stream_encoder_get_do_escape_coding (this->encoder));
-#endif
       break;
     case PROP_EXHAUSTIVE_MODEL_SEARCH:
-#ifdef LEGACY_FLAC
-      g_value_set_boolean (value,
-          FLAC__seekable_stream_encoder_get_do_exhaustive_model_search
-          (this->encoder));
-#else
       g_value_set_boolean (value,
           FLAC__stream_encoder_get_do_exhaustive_model_search (this->encoder));
-#endif
       break;
     case PROP_MIN_RESIDUAL_PARTITION_ORDER:
-#ifdef LEGACY_FLAC
-      g_value_set_uint (value,
-          FLAC__seekable_stream_encoder_get_min_residual_partition_order
-          (this->encoder));
-#else
       g_value_set_uint (value,
           FLAC__stream_encoder_get_min_residual_partition_order
           (this->encoder));
-#endif
       break;
     case PROP_MAX_RESIDUAL_PARTITION_ORDER:
-#ifdef LEGACY_FLAC
-      g_value_set_uint (value,
-          FLAC__seekable_stream_encoder_get_max_residual_partition_order
-          (this->encoder));
-#else
       g_value_set_uint (value,
           FLAC__stream_encoder_get_max_residual_partition_order
           (this->encoder));
-#endif
       break;
     case PROP_RICE_PARAMETER_SEARCH_DIST:
-#ifdef LEGACY_FLAC
-      g_value_set_uint (value,
-          FLAC__seekable_stream_encoder_get_rice_parameter_search_dist
-          (this->encoder));
-#else
       g_value_set_uint (value,
           FLAC__stream_encoder_get_rice_parameter_search_dist (this->encoder));
-#endif
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1540,19 +1299,11 @@ gst_flac_enc_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-#ifdef LEGACY_FLAC
-      if (FLAC__seekable_stream_encoder_get_state (flacenc->encoder) !=
-          FLAC__STREAM_ENCODER_UNINITIALIZED) {
-        flacenc->stopped = TRUE;
-        FLAC__seekable_stream_encoder_finish (flacenc->encoder);
-      }
-#else
       if (FLAC__stream_encoder_get_state (flacenc->encoder) !=
           FLAC__STREAM_ENCODER_UNINITIALIZED) {
         flacenc->stopped = TRUE;
         FLAC__stream_encoder_finish (flacenc->encoder);
       }
-#endif
       flacenc->offset = 0;
       flacenc->samples_written = 0;
       flacenc->channels = 0;
