@@ -122,7 +122,7 @@ gst_gconf_video_sink_init (GstGConfVideoSink * sink,
   sink->client = gconf_client_get_default ();
   gconf_client_add_dir (sink->client, GST_GCONF_DIR,
       GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
-  gconf_client_notify_add (sink->client,
+  sink->notify_id = gconf_client_notify_add (sink->client,
       GST_GCONF_DIR "/" GST_GCONF_VIDEOSINK_KEY,
       cb_toggle_element, sink, NULL, NULL);
 }
@@ -133,6 +133,9 @@ gst_gconf_video_sink_dispose (GObject * object)
   GstGConfVideoSink *sink = GST_GCONF_VIDEO_SINK (object);
 
   if (sink->client) {
+    if (sink->notify_id != 0)
+      gconf_client_notify_remove (sink->client, sink->notify_id);
+
     g_object_unref (G_OBJECT (sink->client));
     sink->client = NULL;
   }
