@@ -41,6 +41,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include <libavc1394/avc1394.h>
 #include <libavc1394/avc1394_vcr.h>
@@ -317,7 +318,7 @@ gst_hdv1394src_iec61883_receive (unsigned char *data, int len,
     return -1;
 
   if (len == IEC61883_MPEG2_TSP_SIZE) {
-    memcpy (dv1394src->outdata + dv1394src->outoffset, data, len);
+    memcpy ((guint8 *) dv1394src->outdata + dv1394src->outoffset, data, len);
     dv1394src->outoffset += len;
   }
   dv1394src->frame_sequence++;
@@ -480,7 +481,6 @@ gst_hdv1394src_discover_avc_node (GstHDV1394Src * src)
   /* loop over all our ports */
   for (; j < m && node == -1; j++) {
     raw1394handle_t handle;
-    gint n_ports;
     struct raw1394_portinfo pinf[16];
 
     /* open the port */
@@ -489,7 +489,7 @@ gst_hdv1394src_discover_avc_node (GstHDV1394Src * src)
       GST_WARNING ("raw1394 - failed to get handle: %s.\n", strerror (errno));
       continue;
     }
-    if ((n_ports = raw1394_get_port_info (handle, pinf, 16)) < 0) {
+    if (raw1394_get_port_info (handle, pinf, 16) < 0) {
       GST_WARNING ("raw1394 - failed to get port info: %s.\n",
           strerror (errno));
       goto next;
