@@ -47,6 +47,7 @@ struct _GstShout2send {
   gchar *ip;
   guint port;
   gchar *password;
+  gchar *username;
   gchar *streamname;
   gchar *description;
   gchar *genre;
@@ -54,7 +55,8 @@ struct _GstShout2send {
   gchar *url;
   gboolean connected;
   gchar *songmetadata;
-
+  gchar *songartist;
+  gchar *songtitle;
   guint16 audio_format;
 
   GstTagList* tags;
