@@ -1143,16 +1143,12 @@ forward_event_func (GstPad * pad, GValue * ret, GstEvent * event)
 static gboolean
 forward_event (GstInterleave * self, GstEvent * event)
 {
-  gboolean ret;
-
   GstIterator *it;
   GValue vret = { 0 };
 
   GST_LOG_OBJECT (self, "Forwarding event %p (%s)", event,
       GST_EVENT_TYPE_NAME (event));
 
-  ret = TRUE;
-
   g_value_init (&vret, G_TYPE_BOOLEAN);
   g_value_set_boolean (&vret, TRUE);
   it = gst_element_iterate_sink_pads (GST_ELEMENT_CAST (self));
@@ -1161,9 +1157,7 @@ forward_event (GstInterleave * self, GstEvent * event)
   gst_iterator_free (it);
   gst_event_unref (event);
 
-  ret = g_value_get_boolean (&vret);
-
-  return ret;
+  return g_value_get_boolean (&vret);
 }
 
 
