@@ -21,15 +21,16 @@
 
 /**
  * SECTION:element-monoscope
- * @see_also: monoscope
+ * @see_also: goom
+ *
+ * Monoscope is an audio visualisation element. It creates a coloured
+ * curve of the audio signal like on an oscilloscope.
  *
  * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v audiotestsrc ! audioconvert ! monoscope ! ffmpegcolorspace ! ximagesink
- * </programlisting>
- * </para>
+ * ]|
  * </refsect2>
  */
 
@@ -159,6 +160,9 @@ gst_monoscope_init (GstMonoscope * monoscope, GstMonoscopeClass * klass)
   monoscope->fps_num = 25;      /* desired frame rate */
   monoscope->fps_denom = 1;
   monoscope->visstate = NULL;
+
+  /* reset the initial audio state */
+  monoscope->rate = GST_AUDIO_DEF_RATE;
 }
 
 static void
@@ -309,8 +313,8 @@ get_buffer (GstMonoscope * monoscope, GstBuffer ** outbuf)
 static GstFlowReturn
 gst_monoscope_chain (GstPad * pad, GstBuffer * inbuf)
 {
+  GstFlowReturn flow_ret = GST_FLOW_OK;
   GstMonoscope *monoscope;
-  GstFlowReturn flow_ret;
 
   monoscope = GST_MONOSCOPE (GST_PAD_PARENT (pad));
 
@@ -331,8 +335,6 @@ gst_monoscope_chain (GstPad * pad, GstBuffer * inbuf)
   gst_adapter_push (monoscope->adapter, inbuf);
   inbuf = NULL;
 
-  flow_ret = GST_FLOW_OK;
-
   /* Collect samples until we have enough for an output frame */
   while (flow_ret == GST_FLOW_OK) {
     gint16 *samples;
@@ -516,7 +518,6 @@ gst_monoscope_change_state (GstElement * element, GstStateChange transition)
   GstMonoscope *monoscope = GST_MONOSCOPE (element);
   GstStateChangeReturn ret;
 
-
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
