@@ -163,7 +163,7 @@ GST_START_TEST (test_mono_middle)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
 
   res = (gint16 *) GST_BUFFER_DATA (outbuffer);
-  GST_INFO ("expected %+5ld %+5ld %+5ld %+5ld real %+5ld %+5ld %+5ld %+5ld",
+  GST_INFO ("expected %+5d %+5d %+5d %+5d real %+5d %+5d %+5d %+5d",
       out[0], out[1], out[2], out[3], res[0], res[1], res[2], res[3]);
   fail_unless (memcmp (GST_BUFFER_DATA (outbuffer), out, 8) == 0);
 
@@ -203,7 +203,7 @@ GST_START_TEST (test_mono_left)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
 
   res = (gint16 *) GST_BUFFER_DATA (outbuffer);
-  GST_INFO ("expected %+5ld %+5ld %+5ld %+5ld real %+5ld %+5ld %+5ld %+5ld",
+  GST_INFO ("expected %+5d %+5d %+5d %+5d real %+5d %+5d %+5d %+5d",
       out[0], out[1], out[2], out[3], res[0], res[1], res[2], res[3]);
   fail_unless (memcmp (GST_BUFFER_DATA (outbuffer), out, 8) == 0);
 
@@ -243,7 +243,7 @@ GST_START_TEST (test_mono_right)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
 
   res = (gint16 *) GST_BUFFER_DATA (outbuffer);
-  GST_INFO ("expected %+5ld %+5ld %+5ld %+5ld real %+5ld %+5ld %+5ld %+5ld",
+  GST_INFO ("expected %+5d %+5d %+5d %+5d real %+5d %+5d %+5d %+5d",
       out[0], out[1], out[2], out[3], res[0], res[1], res[2], res[3]);
   fail_unless (memcmp (GST_BUFFER_DATA (outbuffer), out, 8) == 0);
 
@@ -282,7 +282,7 @@ GST_START_TEST (test_stereo_middle)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
 
   res = (gint16 *) GST_BUFFER_DATA (inbuffer);
-  GST_INFO ("expected %+5ld %+5ld %+5ld %+5ld real %+5ld %+5ld %+5ld %+5ld",
+  GST_INFO ("expected %+5d %+5d %+5d %+5d real %+5d %+5d %+5d %+5d",
       in[0], in[1], in[2], in[3], res[0], res[1], res[2], res[3]);
   fail_unless (memcmp (GST_BUFFER_DATA (outbuffer), in, 8) == 0);
 
@@ -323,7 +323,7 @@ GST_START_TEST (test_stereo_left)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
 
   res = (gint16 *) GST_BUFFER_DATA (outbuffer);
-  GST_INFO ("expected %+5ld %+5ld %+5ld %+5ld real %+5ld %+5ld %+5ld %+5ld",
+  GST_INFO ("expected %+5d %+5d %+5d %+5d real %+5d %+5d %+5d %+5d",
       out[0], out[1], out[2], out[3], res[0], res[1], res[2], res[3]);
   fail_unless (memcmp (GST_BUFFER_DATA (outbuffer), out, 8) == 0);
 
@@ -363,7 +363,7 @@ GST_START_TEST (test_stereo_right)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
 
   res = (gint16 *) GST_BUFFER_DATA (outbuffer);
-  GST_INFO ("expected %+5ld %+5ld %+5ld %+5ld real %+5ld %+5ld %+5ld %+5ld",
+  GST_INFO ("expected %+5d %+5d %+5d %+5d real %+5d %+5d %+5d %+5d",
       out[0], out[1], out[2], out[3], res[0], res[1], res[2], res[3]);
   fail_unless (memcmp (GST_BUFFER_DATA (outbuffer), out, 8) == 0);
 
@@ -402,7 +402,7 @@ GST_START_TEST (test_mono_middle_simple)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
 
   res = (gint16 *) GST_BUFFER_DATA (outbuffer);
-  GST_INFO ("expected %+5ld %+5ld %+5ld %+5ld real %+5ld %+5ld %+5ld %+5ld",
+  GST_INFO ("expected %+5d %+5d %+5d %+5d real %+5d %+5d %+5d %+5d",
       out[0], out[1], out[2], out[3], res[0], res[1], res[2], res[3]);
   fail_unless (memcmp (GST_BUFFER_DATA (outbuffer), out, 8) == 0);
 
@@ -443,7 +443,7 @@ GST_START_TEST (test_mono_left_simple)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
 
   res = (gint16 *) GST_BUFFER_DATA (outbuffer);
-  GST_INFO ("expected %+5ld %+5ld %+5ld %+5ld real %+5ld %+5ld %+5ld %+5ld",
+  GST_INFO ("expected %+5d %+5d %+5d %+5d real %+5d %+5d %+5d %+5d",
       out[0], out[1], out[2], out[3], res[0], res[1], res[2], res[3]);
   fail_unless (memcmp (GST_BUFFER_DATA (outbuffer), out, 8) == 0);
 
@@ -484,7 +484,7 @@ GST_START_TEST (test_mono_right_simple)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
 
   res = (gint16 *) GST_BUFFER_DATA (outbuffer);
-  GST_INFO ("expected %+5ld %+5ld %+5ld %+5ld real %+5ld %+5ld %+5ld %+5ld",
+  GST_INFO ("expected %+5d %+5d %+5d %+5d real %+5d %+5d %+5d %+5d",
       out[0], out[1], out[2], out[3], res[0], res[1], res[2], res[3]);
   fail_unless (memcmp (GST_BUFFER_DATA (outbuffer), out, 8) == 0);
 
@@ -524,7 +524,7 @@ GST_START_TEST (test_stereo_middle_simple)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
 
   res = (gint16 *) GST_BUFFER_DATA (inbuffer);
-  GST_INFO ("expected %+5ld %+5ld %+5ld %+5ld real %+5ld %+5ld %+5ld %+5ld",
+  GST_INFO ("expected %+5d %+5d %+5d %+5d real %+5d %+5d %+5d %+5d",
       in[0], in[1], in[2], in[3], res[0], res[1], res[2], res[3]);
   fail_unless (memcmp (GST_BUFFER_DATA (outbuffer), in, 8) == 0);
 
@@ -566,7 +566,7 @@ GST_START_TEST (test_stereo_left_simple)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
 
   res = (gint16 *) GST_BUFFER_DATA (outbuffer);
-  GST_INFO ("expected %+5ld %+5ld %+5ld %+5ld real %+5ld %+5ld %+5ld %+5ld",
+  GST_INFO ("expected %+5d %+5d %+5d %+5d real %+5d %+5d %+5d %+5d",
       out[0], out[1], out[2], out[3], res[0], res[1], res[2], res[3]);
   fail_unless (memcmp (GST_BUFFER_DATA (outbuffer), out, 8) == 0);
 
@@ -607,7 +607,7 @@ GST_START_TEST (test_stereo_right_simple)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
 
   res = (gint16 *) GST_BUFFER_DATA (outbuffer);
-  GST_INFO ("expected %+5ld %+5ld %+5ld %+5ld real %+5ld %+5ld %+5ld %+5ld",
+  GST_INFO ("expected %+5d %+5d %+5d %+5d real %+5d %+5d %+5d %+5d",
       out[0], out[1], out[2], out[3], res[0], res[1], res[2], res[3]);
   fail_unless (memcmp (GST_BUFFER_DATA (outbuffer), out, 8) == 0);
 
